from numtoname.functions import generate_name_fixed
from numtoname.functions import generate_names_fixed
from numtoname.functions import generate_name
from numtoname.functions import generate_names
from numtoname.functions import generate_name_fixed_alpha
from numtoname.functions import generate_names_fixed_alpha
from numtoname.functions import generate_name_alpha
from numtoname.functions import generate_names_alpha
from numtoname.functions import generate_name_fixed_alpha2
from numtoname.functions import generate_names_fixed_alpha2
from numtoname.functions import generate_name_alpha2
from numtoname.functions import generate_names_alpha2



# A python module to convert a number or list of numbers into a variable name or list of variable names

def generate_name_fixed(num: int, char_order: str, char_length: int):
    if num < 1 or char_order is None or len(char_order) < 1 or char_length < 1:
        return ''
    
    name_string = ""
    running_length = num
    char_num = len(char_order)
    for i in range(char_length):
        if running_length > 1 and running_length > (char_num ** (char_length - i - 1)):
            for j in range(char_num):
                if running_length > ((char_num - j - 1) * (char_num ** (char_length - i - 1))):
                    name_string += char_order[(char_num - j - 1)]
                    running_length -= ((char_num - j - 1) * (char_num ** (char_length - i - 1)))
                    break
        else:
            name_string += char_order[0]
    
    return name_string


def generate_names_fixed(start_num: int, end_num: int, char_order: str, char_length: int):
    if start_num < 1 or start_num > end_num or char_order is None or len(char_order) < 1 or char_length < 1:
        return []
    
    names = []
    for i in range(start_num, end_num + 1):
        names.append(generate_name_fixed(i, char_order, char_length))
    
    return names


def generate_name(num: int, char_order: str):
    if num < 1 or char_order is None or len(char_order) < 1:
        return ''

    char_length = 0
    running_total = 0
    last_running_total = 0
    while True:
        if num > running_total:
            char_length += 1
            last_running_total = running_total
            running_total += len(char_order) ** char_length
        else:
            break
    
    return generate_name_fixed(num - last_running_total, char_order, char_length)


def generate_names(start_num: int, end_num: int, char_order: str):
    if start_num < 1 or start_num > end_num or char_order is None or len(char_order) < 1:
        return []
    
    names = []
    for i in range(start_num, end_num + 1):
        names.append(generate_name(i, char_order))
    
    return names


def generate_name_fixed_alpha(num: int, char_length: int):
    char_order = 'abcdefghijklmnopqrstuvwxyz'
    return generate_name_fixed(num, char_order, char_length)


def generate_names_fixed_alpha(start_num: int, end_num: int, char_length: int):
    char_order = 'abcdefghijklmnopqrstuvwxyz'
    return generate_names_fixed(start_num, end_num, char_order, char_length)
    

def generate_name_alpha(num: int):
    char_order = 'abcdefghijklmnopqrstuvwxyz'
    return generate_name(num, char_order)


def generate_names_alpha(start_num: int, end_num: int):
    char_order = 'abcdefghijklmnopqrstuvwxyz'
    return generate_names(start_num, end_num, char_order)


def generate_name_fixed_alpha2(num: int, char_length: int):
    char_order = 'aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ'
    return generate_name_fixed(num, char_order, char_length)


def generate_names_fixed_alpha2(start_num: int, end_num: int, char_length: int):
    char_order = 'aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ'
    return generate_names_fixed(start_num, end_num, char_order, char_length)
    

def generate_name_alpha2(num: int):
    char_order = 'aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ'
    return generate_name(num, char_order)


def generate_names_alpha2(start_num: int, end_num: int):
    char_order = 'aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ'
    return generate_names(start_num, end_num, char_order)





from distutils.core import setup
setup(
  name = 'numtoname',         # How you named your package folder (MyLib)
  packages = ['numtoname'],   # Chose the same as "name"
  version = '0.1.3',
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'A python module to convert a number or list of numbers into a variable name or list of variable names',
  author = 'Ben Messer',
  author_email = 'benjamin.messer@outlook.com',
  url = 'https://github.com/benjaminmesser/Numtoname',
  download_url = 'https://github.com/benjaminmesser/Numtoname/archive/v_013.tar.gz',
  keywords = ['tools', 'development', 'naming'],
  install_requires=[],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
  ],
)



import functions


char_order1 = 'aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ'
char_order2 = 'abcdefghijklmnopqrstuvwxyz'
char_order3 = 'abcdefgh'


def test_generate_name_fixed():
    assert functions.generate_name_fixed(123123, char_order1, 10) == 'aaaaaaaWNt'
    assert functions.generate_name_fixed(1, char_order1, 10) == 'aaaaaaaaaa'
    assert functions.generate_name_fixed(1313142525, char_order1, 10) == "aaaaBLPAqw"
    assert functions.generate_name_fixed(12, char_order1, 10) == 'aaaaaaaaaF'
    assert functions.generate_name_fixed(52, char_order1, 10) == 'aaaaaaaaaZ'
    assert functions.generate_name_fixed(54, char_order1, 10) == 'aaaaaaaaAA'
    assert functions.generate_name_fixed(2, char_order1, 10) == 'aaaaaaaaaA'
    assert functions.generate_name_fixed(104, char_order1, 10) == 'aaaaaaaaAZ'
    assert functions.generate_name_fixed(105, char_order1, 10) == 'aaaaaaaaba'
    assert functions.generate_name_fixed(53, char_order1, 10) == 'aaaaaaaaAa'
    assert functions.generate_name_fixed(2704, char_order1, 10) == 'aaaaaaaaZZ'
    assert functions.generate_name_fixed(2705, char_order1, 10) == 'aaaaaaaAaa'
    assert functions.generate_name_fixed(2709, char_order1, 10) == 'aaaaaaaAac'
    assert functions.generate_name_fixed(676, char_order2, 5) == 'aaazz'
    assert functions.generate_name_fixed(677, char_order2, 5) == 'aabaa'
    assert functions.generate_name_fixed(0, char_order2, 5) == ''
    assert functions.generate_name_fixed(50, char_order2, 0) == ''
    assert functions.generate_name_fixed(-5, char_order2, 5) == ''
    assert functions.generate_name_fixed(50, '', 5) == ''
    assert functions.generate_name_fixed(50, None, 5) == ''


def test_generate_name():
    assert functions.generate_name(2, char_order3) == 'b'
    assert functions.generate_name(1, char_order3) == 'a'
    assert functions.generate_name(9, char_order3) == 'aa'
    assert functions.generate_name(11, char_order3) == 'ac'
    assert functions.generate_name(73, char_order3) == 'aaa'
    assert functions.generate_name(0, char_order3) == ''
    assert functions.generate_name(-6, char_order3) == ''
    assert functions.generate_name(5, '') == ''
    assert functions.generate_name(5, None) == ''


def test_generate_names_fixed():
    assert functions.generate_names_fixed(1, 5, char_order3, 5) == ['aaaaa', 'aaaab', 'aaaac', 'aaaad', 'aaaae']
    assert functions.generate_names_fixed(11, 14, char_order3, 5) == ['aaabc', 'aaabd', 'aaabe', 'aaabf']
    assert functions.generate_names_fixed(14, 13, char_order3, 5) == []
    assert functions.generate_names_fixed(0, 14, char_order3, 5) == []
    assert functions.generate_names_fixed(-5, 14, char_order3, 5) == []
    assert functions.generate_names_fixed(10, 14, char_order3, 0) == []
    assert functions.generate_names_fixed(10, 14, '', 5) == []
    assert functions.generate_names_fixed(10, 14, None, 5) == []


def test_generate_names():
    assert functions.generate_names(1, 5, char_order3) == ['a', 'b', 'c', 'd', 'e']
    assert functions.generate_names(11, 14, char_order3) == ['ac', 'ad', 'ae', 'af']
    assert functions.generate_names(14, 13, char_order3) == []
    assert functions.generate_names(0, 14, char_order3) == []
    assert functions.generate_names(-5, 14, char_order3) == []
    assert functions.generate_names(10, 14, '') == []
    assert functions.generate_names(10, 14, None) == []


if __name__ == '__main__':
    test_generate_name_fixed()
    test_generate_name()
    test_generate_names_fixed()
    test_generate_names()

    print('All tests passed successfully!')

